// hpstat: High-performance statistics implementations
// Copyright © 2023  Lee Yingtong Li (RunasSudo)
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
// 
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

use std::io;

use console::Term;
use indicatif::TermLike;

#[derive(Debug)]
pub struct UnconditionalTermLike {
	term: Term
}

impl UnconditionalTermLike {
	//pub fn new(term: Term) -> UnconditionalTermLike {
	//	return UnconditionalTermLike { term: term };
	//}
	
	pub fn stderr() -> UnconditionalTermLike {
		return UnconditionalTermLike { term: Term::stderr() };
	}
}

impl TermLike for UnconditionalTermLike {
	fn width(&self) -> u16 {
		return self.term.width();
	}
	
	fn move_cursor_up(&self, n: usize) -> io::Result<()> {
		return self.term.move_cursor_up(n);
	}
	
	fn move_cursor_down(&self, n: usize) -> io::Result<()> {
		return self.term.move_cursor_down(n);
	}
	
	fn move_cursor_right(&self, n: usize) -> io::Result<()> {
		return self.term.move_cursor_right(n);
	}
	
	fn move_cursor_left(&self, n: usize) -> io::Result<()> {
		return self.term.move_cursor_left(n);
	}
	
	fn write_line(&self, s: &str) -> io::Result<()> {
		return self.term.write_line(s);
	}
	
	fn write_str(&self, s: &str) -> io::Result<()> {
		return self.term.write_str(s);
	}
	
	fn clear_line(&self) -> io::Result<()> {
		return self.term.clear_line();
	}
	
	fn flush(&self) -> io::Result<()> {
		return self.term.flush();
	}
	
}
